%
% m-file:      CatenaryResponse.m
%
% Purpose:     Model the response of a catenary system 

% Note:        This file is for running the numerical model, sub routines
%              are invoked and can be examined separately.  Using the
%              programs that plot results, it is best to implement the
%              reference system described in the thesis.  This is because
%              some of the plots and results are connected to certain
%              degrees of freedom that will change if the span length or
%              number of spans is changed.  My suggestion is to run it with
%              the reference system first to see how it works and then
%              implementing changes.  If the purpose is to model a scaled
%              system, most of the parameters that need to be multiplied by
%              a scaling factors are found in INPUT VARIABLES.  In
%              addition, variables in WBL88 and the friction force of the
%              pantograph in Newmarkint need to be scaled.
%
% Written by:  Arnar Kari Hallgrimsson, Spring 2013
%

close all
clear all
clc
tic

%% INPUT VARIABLES

%--- Contact wire (CW) ----------------------------------------------------

E_CW = 1.24e11;            %Young's modulus [N/m^2]
I_CW = 9.87e-10;           %Moment of inertia [m^4]
RHO_CW = 0.94;             %Mass per unit length [kg/m]
N_CW = -10e3;              %Tension force [N] (-) for tension
xi_CW = 0.01;              %Critical damping ratio
A_CW = 105.7e-6;           %Cross sectional area [m^2]

%--- Messenger wire (MW) --------------------------------------------------

E_MW = 1.13e11;            %Young's modulus [N/m^2]
I_MW = 4.935e-10;          %Moment of inertia [m^4]
RHO_MW = 0.46;             %Mass per unit length [kg/m]
N_MW = -5e3;               %Tension force [N] (-) for tension          
xi_MW = 0.01;              %Critical damping ratio
A_MW = 50e-6;              %Cross sectional area [m^2]

%--- Reg arm, support bracket and droppers --------------------------------

DROPP_K = 1e5;             %Stiffness [N/m]
DROPP_M = 0.2;             %Mass [kg]
DROPP_C = 5;               %Damping [Ns/m]
DROPP_LOC0 = [2 9 16.5 23.5 31 38]; %Dropper location in a span [m]
DROPP_HI0 = [1.48 1.36 1.29 1.29 1.36 1.48]; %Droppers height [m]
DROPP_HI = [1.6 1.48 1.36 1.29 1.29 1.36 1.48 1.6]; %Dropper and support 
                                                    %pole height

REG_K = 213;               %Stiffness of registration arm [N/m]
REG_M = 2.6;               %Mass og registration arm [kg]

SUPP_K = 2e6;               %Stiffness of support bracket [N/m]
SUPP_M = 2.6;               %Mass of support bracket [kg]
SUPP_MX = 2.6;              %Mass of support bracket in x-direction [kg]
SUPP_KX = 2e6;              %Stiffness of support bracket in x-direction [N/m]

%--- Model geometry -------------------------------------------------------

L_EL = 0.5;                 %Length of element [m]
SPANS = 4;                  %Nr. of modelled spans
L = 40;                     %Lenght of a span
vel1 = 100;                 %Velocity of train [km/h]
dt = 0.001;                 %Time step for integration method
kc = 50000;                 %Contact stiffness
g = -9.81;                  %Acceleration of gravity [m/s^2]
xiw = [0.01 0.04];          %Wilson-Penzien damping boundaries

%% Derived variables for the analyses

L_TOT = SPANS*L;            %Total length
NOFN = (L_TOT/L_EL+1)*2;    %Nr. of nodes
NOE = NOFN-2;               %Nr. of elements
NDOF = NOFN+NOFN/2*3;       %Nr. of degrees of freedom
vel = vel1*1e3/3600;        %Velocity of train [m/s]
Tpass = L_TOT/vel;          %Time it takes the train to pass a span
panto = (L_TOT/2)/vel;      %Time it takes pantograph to reach x = 25 m

%--- Build vector containing locations of all droppers --------------------

n_dr1 = length(DROPP_LOC0);
DROPP_HI1 = repmat(DROPP_HI0,1,SPANS);
add = 0;
nr1 = 1;
for i = 1:n_dr1:n_dr1*SPANS
    DROPP_LOC(i:n_dr1*nr1) = DROPP_LOC0+add;
    add = add+L;
    nr1 = nr1+1;
end

%--- Assemble problem parameters in one place for functions ---------------

EI = [E_CW*I_CW E_MW*I_MW];
N = [N_CW N_MW];
RHO = [RHO_CW RHO_MW];
IK = [REG_K DROPP_K SUPP_K SUPP_KX];
IM = [REG_M DROPP_M SUPP_M SUPP_MX];
E = [E_CW E_MW];
A = [A_CW A_MW];
xi = [xi_CW xi_MW];

%% ANALYSES

%--- Assemble geometry ----------------------------------------------------

[x,y] = Geometry(NOE,SPANS,L_TOT,DROPP_LOC0,DROPP_HI);

%--- Assemble mass and stiffness matrices for catenary ---

[KG,MG,FG] = assembleCat(L_EL,EI,N,RHO,NOE,NOFN,y,E,A,NDOF,g);

KC = KG;
MC = MG;

KC = sparse(KC);
MC = sparse(MC);

%--- Additions from reg arm, support bracket and droppers -----------------

[KG,MG,FG,n_dropp,n_dropp1] = indadd(KG,MG,FG,DROPP_LOC,NDOF,...
    IK,IM,x,NOE,SPANS,NOFN,g);

%--- Eigenfrequencies and modes -------------------------------------------

[V,D] = eig(KG,MG);
EIGFREQ = sqrt(diag(D))/(2*pi());  %Eigenfrequencies [Hz]
omeg = sqrt(diag(D));              %Eigenfrequencies [rad/s]

%--- Damping matrix -------------------------------------------------------

%-- Here you can choose between Rayleigh damping and Wilson&Penzien damping
%-- Just uncomment one and comment the other.

[CG] = Rayleighdamping(omeg,xi,MC,KC,n_dropp,n_dropp1,DROPP_C,NOFN,NDOF);

% [CG] = WilsonPenziendamping(omeg,V,MC,xiw,n_dropp,n_dropp1,DROPP_C,NDOF);

KG = sparse(KG);
MG = sparse(MG);
CG = sparse(CG);

%--- Pantograph -----------------------------------------------------------

[MG,CG,KG,F0] = WBL88(MG,CG,KG);

%--- Static response ------------------------------------------------------

D_stat = zeros(length(CG),1);
D_stat(1:(end-2),1) = KG(1:(end-2),1:(end-2))\FG;
D_stat(1:2:NOFN) = D_stat(1:2:NOFN)-D_stat(1,1);

%--- Direct integration method --------------------------------------------

[P,Dall,D_dyn,count,xp,t] =  Newmarkint(MG,CG,KG,dt,Tpass,F0,NOFN,...
    NDOF,N,D_stat,vel,L_TOT,L_EL,kc);


%% RESULTS

tempo = 1;
while tempo==1

flag = menu('Which plot do you want','Themodel','Staticdisplacement',...
    'Eigenfrequencies','Elasticity','Dropperdisplacement',...
    'PantographPosition','Dynamicthreepoints','Contactforce','Motion','Exit');

switch flag
    case 1
    Themodel % Plots up the initial geometry of the model

    case 2
    Staticdisplacement % Plots up initial geometry and static displacement
                       % of whole model and span 3

    case 3
    Eigenfrequencies   % Plots up first 5 eigenfrequencies, 10th, 20th and 
                       % the 100th
    
    case 4
    Elasticity         % Plots up elasticity from the model with measured
                       % elasticity

    case 5
    Dropperdisplacement     % Plots vertical displacement of droppers 15 
                            % and 18

    case 6
    PantographPosition      % Plots pantograph vertical displacement and
                            % vertical displacement of contact wire at that
                            % the same points, when they are together.
    
    case 7
    Dynamicthreepoints      % Plots up vertical displacement of three 
                            % different points at x=80m,90m,100m
 
    case 8
    Contactforce            % Plots up contact force for all spans and just
                            % span 3
    
    case 9
    Motion
    
    case 10
    tempo = 0;
end

end





toc